class Monod:
    def __init__(self, max_uptake_rate: float, affinity_constant: float) -> None:
        self.max_uptake_rate = max_uptake_rate
        self.affinity_constant = affinity_constant

    def calculate_rate(self, substrate_concentration: float) -> float:
        return (
            self.max_uptake_rate
            * substrate_concentration
            / (self.affinity_constant + substrate_concentration)
        )


class MonodSubstrateInhibition:
    def __init__(
        self,
        max_uptake_rate: float,
        affinity_constant: float,
        inhibition_constant: float,
    ) -> None:
        self.max_uptake_rate = max_uptake_rate
        self.affinity_constant = affinity_constant
        self.inhibition_constant = inhibition_constant

    def calculate_rate(self, substrate_concentration: float) -> float:
        return (
            self.max_uptake_rate
            * substrate_concentration
            / (
                self.affinity_constant
                + substrate_concentration
                + (substrate_concentration**2) / self.inhibition_constant
            )
        )


class MonodSubstrateCompetitiveInhibition:
    def __init__(
        self,
        max_uptake_rate: float,
        affinity_constant: float,
        inhibition_constant: float,
    ) -> None:
        self.max_uptake_rate = max_uptake_rate
        self.affinity_constant = affinity_constant
        self.inhibition_constant = inhibition_constant

    def calculate_rate(
        self,
        substrate_concentration: float,
    ) -> float:
        return self.max_uptake_rate / (
            (1.0 + self.affinity_constant / substrate_concentration)
            * (1.0 + substrate_concentration / self.inhibition_constant)
        )


class MonodSubstrateNonCompetitiveInhibition:
    def __init__(
        self,
        max_uptake_rate: float,
        affinity_constant: float,
        inhibition_constant: float,
    ) -> None:
        self.max_uptake_rate = max_uptake_rate
        self.affinity_constant = affinity_constant
        self.inhibition_constant = inhibition_constant

    def calculate_rate(self, substrate_concentration: float) -> float:
        return (
            self.max_uptake_rate
            * substrate_concentration
            / (
                self.affinity_constant
                * (1.0 + substrate_concentration / self.inhibition_constant)
                + substrate_concentration
            )
        )



import kineticmodels as km
import pytest


def test_monod_rate():
    # Arrange
    substrate_concentration = 0.5
    max_uptake_rate = 1
    affinity_constant = 0.5

    # Act
    kinetics = km.Monod(max_uptake_rate, affinity_constant)
    rate = kinetics.rate(substrate_concentration)

    # Assert
    assert pytest.approx(rate, 1e-8) == 0.5


def test_monod_rate_from_dict():
    # Arrange
    substrate_concentration = 0.5
    parameters = {
        "max_uptake_rate": 1,
        "affinity_constant": 0.5,
    }

    # Act
    kinetics = km.Monod.fromdict(parameters)
    rate = kinetics.rate(substrate_concentration)

    # Assert
    assert pytest.approx(rate, 1e-8) == 0.5


def test_monod_substrate_inhibition_rate():
    # Arrange
    substrate_concentration = 1.0
    max_uptake_rate = 0.5
    affinity_constant = 0.5
    substrate_inhibition_constant = 1.0

    # Act
    kinetics = km.MonodSubstrateInhibition(
        max_uptake_rate,
        affinity_constant,
        substrate_inhibition_constant,
    )
    rate = kinetics.rate(substrate_concentration)

    # Assert
    assert pytest.approx(rate, 1e-8) == 0.2


def test_monod_substrate_inhibition_rate_from_dict():
    # Arrange
    substrate_concentration = 1.0
    parameters = {
        "max_uptake_rate": 0.5,
        "affinity_constant": 0.5,
        "substrate_inhibition_constant": 1.0,
    }

    # Act
    kinetics = km.MonodSubstrateInhibition.fromdict(parameters)
    rate = kinetics.rate(substrate_concentration)

    # Assert
    assert pytest.approx(rate, 1e-8) == 0.2


def test_monod_substrate_competitive_inhibition_rate():
    # Arrange
    substrate_concentration = 0.5
    max_uptake_rate = 1.0
    affinity_constant = 0.5
    substrate_inhibition_constant = 1.0

    # Act
    kinetics = km.MonodSubstrateCompetitiveInhibition(
        max_uptake_rate,
        affinity_constant,
        substrate_inhibition_constant,
    )
    rate = kinetics.rate(substrate_concentration=substrate_concentration)

    # Assert
    assert pytest.approx(rate, 1e-8) == 0.3333333333333333


def test_monod_substrate_competitive_inhibition_rate_fom_dict():
    # Arrange
    substrate_concentration = 0.5
    parameters = {
        "max_uptake_rate": 1.0,
        "affinity_constant": 0.5,
        "substrate_inhibition_constant": 1.0,
    }

    # Act
    kinetics = km.MonodSubstrateCompetitiveInhibition.fromdict(parameters)
    rate = kinetics.rate(substrate_concentration=substrate_concentration)

    # Assert
    assert pytest.approx(rate, 1e-8) == 0.3333333333333333


def test_monod_substrate_non_competitive_inhibition_rate():
    # Arrange
    substrate_concentration = 0.5
    max_uptake_rate = 1.0
    affinity_constant = 0.5
    substrate_inhibition_constant = 1.0

    # Act
    kinetics = km.MonodSubstrateNonCompetitiveInhibition(
        max_uptake_rate=max_uptake_rate,
        affinity_constant=affinity_constant,
        substrate_inhibition_constant=substrate_inhibition_constant,
    )
    rate = kinetics.rate(substrate_concentration)
    # Assert
    assert pytest.approx(rate, 1e-8) == 0.4


def test_monod_substrate_non_competitive_inhibition_rate_dict():
    # Arrange
    substrate_concentration = 0.5
    parameters = {
        "max_uptake_rate": 1.0,
        "affinity_constant": 0.5,
        "substrate_inhibition_constant": 1.0,
    }

    # Act
    kinetics = km.MonodSubstrateNonCompetitiveInhibition.fromdict(parameters)
    rate = kinetics.rate(substrate_concentration)
    # Assert
    assert pytest.approx(rate, 1e-8) == 0.4



from kineticmodels.interfaces import IMonod, IMonodInhibition


class Monod(IMonod):
    """
    Monod uptake model.
    """

    @classmethod
    def fromdict(cls, d: dict) -> "Monod":
        return cls(**d)

    def rate(self, substrate_concentration: float) -> float:
        return (
            self.max_uptake_rate
            * substrate_concentration
            / (self.affinity_constant + substrate_concentration)
        )


class MonodSubstrateInhibition(IMonodInhibition):
    """
    Monod uptake model with substrate inhibition.
    """

    @classmethod
    def fromdict(cls, d: dict) -> "MonodSubstrateInhibition":
        return cls(**d)

    def rate(self, substrate_concentration: float) -> float:
        return (
            self.max_uptake_rate
            * substrate_concentration
            / (
                self.affinity_constant
                + substrate_concentration
                + (substrate_concentration**2) / self.substrate_inhibition_constant
            )
        )


class MonodSubstrateCompetitiveInhibition(IMonodInhibition):
    """
    Monod uptake model with competitive substrate inhibition.
    """

    @classmethod
    def fromdict(cls, d: dict) -> "MonodSubstrateCompetitiveInhibition":
        return cls(**d)

    def rate(
        self,
        substrate_concentration: float,
    ) -> float:
        return self.max_uptake_rate / (
            (1.0 + self.affinity_constant / substrate_concentration)
            * (1.0 + substrate_concentration / self.substrate_inhibition_constant)
        )


class MonodSubstrateNonCompetitiveInhibition(IMonodInhibition):
    """
    Monod uptake model with non-competitive substrate inhibition.
    """

    @classmethod
    def fromdict(cls, d: dict) -> "MonodSubstrateNonCompetitiveInhibition":
        return cls(**d)

    def rate(self, substrate_concentration: float) -> float:
        return (
            self.max_uptake_rate
            * substrate_concentration
            / (
                self.affinity_constant
                * (1.0 + substrate_concentration / self.substrate_inhibition_constant)
                + substrate_concentration
            )
        )



from dataclasses import dataclass


@dataclass
class IMonod:
    """Interface for monod uptake models."""

    max_uptake_rate: float
    affinity_constant: float

    def rate(self, substrate_concentration: float) -> float:
        """Calculate the rate of uptake."""
        pass


@dataclass
class IMonodInhibition(IMonod):
    """Interface for monod inhibition uptake models."""

    substrate_inhibition_constant: float



from .uptake_models import Monod  # noqa: F401
from .uptake_models import MonodSubstrateInhibition  # noqa: F401
from .uptake_models import MonodSubstrateCompetitiveInhibition  # noqa: F401
from .uptake_models import MonodSubstrateNonCompetitiveInhibition  # noqa: F401



import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="kineticmodels",
    version_config=True,
    setup_requires=["setuptools-git-versioning"],
    author="Pau Cabaneros",
    author_email="pau.cabaneros@gmail.com",
    description="tool including most used function.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/paucablop/fermentation.kinetic.models",
    project_urls={
        "Bug Tracker": "https://github.com/paucablop/fermentation.kinetic.models/issues/",
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    package_dir={"": "."},
    packages=setuptools.find_packages(where="."),
    python_requires=">=3.7",
    install_requires=[
        "numpy",
        "pytest",
    ],
)
